# -*- coding: utf-8 -*-
__author__ = "Daniel Scheffler"


import numpy as np
try:
    from osgeo import gdal
except ImportError:
    import gdal



# dictionary to translate Numpy data types (strings) into corresponding GDAL data types,
#  e.g. dTypeDic_NumPy2GDAL(str(np.dtype(np.uint8)))
dTypeDic_NumPy2GDAL = {'bool'    : gdal.GDT_Byte,
                       'bool_'   : gdal.GDT_Int32,
                       'int'     : gdal.GDT_Int32,
                       'int8'    : gdal.GDT_Int16,
                       'uint8'   : gdal.GDT_Byte,
                       'uint16'  : gdal.GDT_UInt16,
                       'int16'   : gdal.GDT_Int16,
                       'uint32'  : gdal.GDT_UInt32,
                       'int32'   : gdal.GDT_Int32,
                       'int64'   : gdal.GDT_Float64,
                       'float'   : gdal.GDT_Float32,
                       'float16' : gdal.GDT_Float32,
                       'float32' : gdal.GDT_Float32,
                       'float64' : gdal.GDT_Float64
                       }

# dictionary to translate GDAL data types (strings) into corresponding numpy data types
dTypeDic_GDAL2Numpy = {gdal.GDT_Byte    : np.uint8,
                       gdal.GDT_UInt16  : np.uint16,
                       gdal.GDT_Int16   : np.int16,
                       gdal.GDT_UInt32  : np.uint32,
                       gdal.GDT_Int32   : np.int32,
                       gdal.GDT_Float32 : np.float32,
                       gdal.GDT_Float64 : np.float64,
                       }

# dictionary to translate Numpy data types into GDAL compatible Numpy data types
dTypeDic_NumPy2GDALcompatible = \
    dict(zip(dTypeDic_NumPy2GDAL.keys(),
             [dTypeDic_GDAL2Numpy[dTypeDic_NumPy2GDAL[str(np.dtype(NDT))]] for NDT in dTypeDic_NumPy2GDAL.keys()]))

