# -*- coding: utf-8 -*-
__author__ = "Daniel Scheffler"

import numpy as np

# custom
from shapely.geometry import shape, mapping, box

from ..coord_trafo import imYX2mapYX, mapYX2imYX, pixelToMapYX



def shapelyImPoly_to_shapelyMapPoly_withPRJ(shapelyImPoly, gt,prj):
    #ACTUALLY PRJ IS NOT NEEDED BUT THIS FUNCTION RETURNS OTHER VALUES THAN shapelyImPoly_to_shapelyMapPoly
    geojson   = mapping(shapelyImPoly)
    coords    = list(geojson['coordinates'][0])
    coordsYX = pixelToMapYX(coords,geotransform=gt,projection=prj)
    coordsXY = tuple([(i[1],i[0]) for i in coordsYX])
    geojson['coordinates'] = (coordsXY,)
    return shape(geojson)


def shapelyImPoly_to_shapelyMapPoly(shapelyBox, gt):
    xmin, ymin, xmax, ymax = shapelyBox.bounds
    ymax, xmin = imYX2mapYX((ymax, xmin), gt)
    ymin, xmax = imYX2mapYX((ymin, xmax), gt)
    return box(xmin, ymin, xmax, ymax)


def shapelyBox2BoxYX(shapelyBox,coord_type='image'):
    xmin, ymin, xmax, ymax = shapelyBox.bounds
    assert coord_type in ['image','map']
    UL_YX, UR_YX, LR_YX, LL_YX = ((ymin,xmin),(ymin,xmax),(ymax,xmax),(ymax,xmin)) if coord_type=='image' else \
                                 ((ymax,xmin),(ymax,xmax),(ymin,xmax),(ymin,xmin))
    return UL_YX, UR_YX, LR_YX, LL_YX


def get_boxImXY_from_shapelyPoly(shapelyPoly, im_gt):
    # type: (shapely.Polygon,tuple) -> np.ndarray
    """Converts each vertex coordinate of a shapely polygon into image coordinates corresponding to the given
    geotransform without respect to invalid image coordinates. Those must be filtered later.
    :param shapelyPoly:     <shapely.Polygon>
    :param im_gt:           <list> the GDAL geotransform of the target image
    """
    get_coordsArr = lambda shpPoly: np.swapaxes(np.array(shpPoly.exterior.coords.xy), 0, 1)
    coordsArr     = get_coordsArr(shapelyPoly)
    boxImXY       = [mapYX2imYX((Y, X), im_gt) for X, Y in coordsArr.tolist()] # FIXME incompatible to GMS version
    boxImXY       = [(i[1],i[0]) for i in boxImXY]
    return boxImXY


def round_shapelyPoly_coords(shapelyPoly, precision=10,out_dtype=None):
    geojson = mapping(shapelyPoly)
    geojson['coordinates'] = np.round(np.array(geojson['coordinates']), precision)
    if out_dtype:
        geojson['coordinates'] = geojson['coordinates'].astype(out_dtype)
    return shape(geojson)